#include "events.hpp"

Events::Events()
{
	// initialize the coodinates of the string
	// editing insertion point
	InsertionBarX = UNDEFINED_COORDINATE;
	InsertionBarY = UNDEFINED_COORDINATE;
}


int Events::GetKeyPressed()
{
	// wait until a key is pressed, then
	// return its value
	return bioskey(0);
}

int Events::AnyKeysPressed()
{
	// return zero if no keys are ready to
	// be read from the keyboard
	return bioskey(1);
}

void Events::ProcessUserKeys()
{
	// see if any keys are pressed
	if (AnyKeysPressed() ) {

	  // read the next key
	  int key = GetKeyPressed();

	  // see if there is an icon that has the
	  // keyboard focus
	  if (InsertionBarX == UNDEFINED_COORDINATE) return;
	  if (InsertionBarY == UNDEFINED_COORDINATE) return;

	  // turn off the mouse cursor before processing
	  // the keypress
	  mouse. Hide();

	  // let the icon with the input focus process
	  // the key
	  CurrentScreen->KeyTyped(InsertionBarX, InsertionBarY, key);

	  // restore the mouse cursor
	  mouse. Show();
	}
}

void Events::ProcessMouseActions()
{
	static int old_x, old_y;
	static status, old_status;
	static Icon* current_icon;
	int x, y, changes;

	// save the status of the mouse buttons that
	// were processed previously
	old_status = status;

	// get the current mouse button status
	status = mouse. Status(&x, &y);

	// see if mouse was moved with a button pressed
	if (current_icon) {
	  if ( (x != old_x) || (y != old_y) ) {
	    if  (mouse. LeftButtonDown() )
	      // it was: let the icon process the event
	      current_icon->LeftButtonDragged(x - old_x,
					      y - old_y,
					      status);
	  }
	}

	// update the mouse coordinates for next time around
	old_x = x;  old_y = y;

	if (status == old_status)
	  // no new mouse events to process yet
	  return;

	// record the change from last time around
	changes = status ^ old_status;

	if (changes & MOUSE_LEFT_BUTTON) {
	  // the left mouse button was pressed
	  // turn off the mouse cursor before processing
	  mouse. Hide();

	  // save the icon that was clicked on, in case
	  // the mouse is dragged on it
	  current_icon = CurrentScreen->CurrentIcon(x, y);

	  // process the mouse button click
	  CurrentScreen->LeftButtonChanged(x, y, status);

	  // restore the mouse cursor
	  mouse. Show();
	}

	// terminate the program when the RIGHT mouse
	// button is clicked anywhere on the screen
	if (changes & MOUSE_RIGHT_BUTTON) {
	  GraphicsHandler->Terminate();
	  exit(0);
	}
}

long Events::SystemTime()
{
	time_t system_time;
	// read the system time, showing the number
	// of seconds elapsed since 1/1/70
	time(&system_time);

	return (long) system_time;
}

char* Events::CurrentTime(float time_difference)
{
	// get the current time
	long the_time = SystemTime();

	// adjust system time for the difference
	// between time zones
	the_time += time_difference * 3600;

	// convert to the ANSI standard string format:
	// WWW MMM DD hh:mm:ss YYYY\n
	// e.g.  "WED JAN 15 14:52:29 1990\n"
	char* string = ctime(&the_time);

	// skip to the hour field, and truncate the year
	string += 11;
	*(string + 8) = '\0';
	return string;
}

void Events::Beep()
{
	// issue a short sound at 1000 Hz
	SoundOn(ONE_KILOHERTZ, BEEPER_DELAY);
}

void Events::SoundOn(int frequency, unsigned duration)
{
	// ignore invalid frequencies
	if (frequency == 0) return;

	// compute IBM PC timer value
	int timer_value = TIMER_CLOCK / frequency;

	// program hardware timer's frequency
	outp(TIMER_COMMAND_PORT, TIMER_SET_COMMAND);
	outp(TIMER_DATA_PORT, timer_value & 0xff);
	outp(TIMER_DATA_PORT, timer_value >> 8);

	// turn on the speaker, preserving unused bits
	int old = inp(TIMER_CONTROL_PORT);
	outp(TIMER_CONTROL_PORT, old | 3);

	// keep it on for the time specified, or indefinitely
	if (duration == 0) return;
	Wait(duration);
	SoundOff();
}

void Events::SoundOff()
{
	// turn off the speaker, preserving unused bits
	int old = inp(TIMER_CONTROL_PORT);
	outp(TIMER_CONTROL_PORT, old & 0xFC);
}
